C     PROGRAM XAMPLE                                                    POI04700
C                                                                       POI04701
C***********************************************************************POI04702
C                                                                       POI04703
C          VERSION  2  OCTOBER 1976  INCLUDING ERRATA  OCTOBER 1976     POI04704
C                                                                       POI04705
C         DOCUMENTATION FOR THIS PROGRAM IS GIVEN IN                    POI04706
C                                                                       POI04707
C        EFFICIENT FORTRAN SUBPROGRAMS FOR THE SOLUTION OF              POI04708
C            ELLIPTIC PARTIAL DIFFERENTIAL EQUATIONS                    POI04709
C                                                                       POI04710
C                              BY                                       POI04711
C                                                                       POI04712
C          PAUL SWARZTRAUBER   AND  ROLAND SWEET                        POI04713
C                                                                       POI04714
C             TECHNICAL NOTE TN/IA-109   JULY 1975                      POI04715
C                                                                       POI04716
C       NATIONAL CENTER FOR ATMOSPHERIC RESEARCH  BOULDER,COLORADO 80307POI04717
C                                                                       POI04718
C        WHICH IS SPONSORED BY THE NATIONAL SCIENCE FOUNDATION          POI04719
C                                                                       POI04720
C***********************************************************************POI04721
C                                                                       POI04722
C                                                                       POI04723
C     PROGRAM TO ILLUSTRATE THE USE OF PWSCRT.                          POI04724
C                                                                       POI04725
      DIMENSION       F(120,100) ,BDB(81)    ,W(1294)    ,X(101)     ,  POI04726
     1                Y(81)                                             POI04727
C                                                                       POI04728
C     FROM DIMENSION STATEMENT WE GET VALUE OF IDIMF.  ALSO NOTE THAT W POI04729
C     IS DIMENSIONED 6*(N+1) + 8*(M+1).                                 POI04730
C                                                                       POI04731
      IDIMF = 120                                                       POI04732
      INTL = 0                                                          POI04733
      A = 0.                                                            POI04734
      B = 2.                                                            POI04735
      M = 100                                                           POI04736
      MBDCND = 2                                                        POI04737
      C = -1.                                                           POI04738
      D = 3.                                                            POI04739
      N = 80                                                            POI04740
      NBDCND = 0                                                        POI04741
      ELMBDA = -4.                                                      POI04742
C                                                                       POI04743
C     AUXILIARY QUANTITIES.                                             POI04744
C                                                                       POI04745
      PI = 3.14159265358979                                             POI04746
      PIBY2 = PI/2.                                                     POI04747
      PISQ = PI**2                                                      POI04748
      MP1 = M+1                                                         POI04749
      NP1 = N+1                                                         POI04750
C                                                                       POI04751
C     GENERATE AND STORE GRID POINTS FOR THE PURPOSE OF COMPUTING       POI04752
C     BOUNDARY DATA AND THE RIGHT SIDE OF THE HELMHOLTZ EQUATION.       POI04753
C                                                                       POI04754
      DO 101 I=1,MP1                                                    POI04755
         X(I) = FLOAT(I-1)/50.                                          POI04756
  101 CONTINUE                                                          POI04757
      DO 102 J=1,NP1                                                    POI04758
         Y(J) = -1.+FLOAT(J-1)/20.                                      POI04759
  102 CONTINUE                                                          POI04760
C                                                                       POI04761
C     GENERATE BOUNDARY DATA.                                           POI04762
C                                                                       POI04763
      DO 103 J=1,NP1                                                    POI04764
         BDB(J) = 4.*COS((Y(J)+1.)*PIBY2)                               POI04765
  103 CONTINUE                                                          POI04766
C                                                                       POI04767
C     BDA, BDC, AND BDD ARE DUMMY VARIABLES.                            POI04768
C                                                                       POI04769
      DO 104 J=1,NP1                                                    POI04770
         F(1,J) = 0.                                                    POI04771
  104 CONTINUE                                                          POI04772
C                                                                       POI04773
C     GENERATE RIGHT SIDE OF EQUATION.                                  POI04774
C                                                                       POI04775
      DO 106 I=2,MP1                                                    POI04776
         DO 105 J=1,NP1                                                 POI04777
            F(I,J) = (2.-(4.+PISQ/4.)*X(I)**2)*COS((Y(J)+1.)*PIBY2)     POI04778
  105    CONTINUE                                                       POI04779
  106 CONTINUE                                                          POI04780
      CALL PWSCRT (INTL,A,B,M,MBDCND,BDA,BDB,C,D,N,NBDCND,BDC,BDD,      POI04781
     1             ELMBDA,F,IDIMF,PERTRB,IERROR,W)                      POI04782
C                                                                       POI04783
C     COMPUTE DISCRETIZATION ERROR.  THE EXACT SOLUTION IS              POI04784
C                U(X,Y) = X**2*COS((Y+1)*PIBY2)                         POI04785
C                                                                       POI04786
      ERR = 0.                                                          POI04787
      DO 108 I=1,MP1                                                    POI04788
         DO 107 J=1,NP1                                                 POI04789
            Z = ABS(F(I,J)-X(I)**2*COS((Y(J)+1.)*PIBY2))                POI04790
            IF (Z .GT. ERR) ERR = Z                                     POI04791
  107    CONTINUE                                                       POI04792
  108 CONTINUE                                                          POI04793
      PRINT 1001 , IERROR,ERR                                           POI04794
C                                                                       POI04795
c     # output solution
      write(61,*) mp1,np1
      do i=1,mp1
         do j=1,np1
            write(66,*) f(i,j),
     &                  X(I)**2*COS((Y(J)+1.)*PIBY2)
            enddo
         enddo
C                                                                       POI04796
C                                                                       POI04797
 1001 FORMAT (///9H IERROR =,I2,10X,22HDISCRETIZATION ERROR =,E12.5)    POI04798
C                                                                       POI04799
      END                                                               POI04800
