C     PROGRAM XAMPLE                                                    POI04802
C                                                                       POI04803
C***********************************************************************POI04804
C                                                                       POI04805
C          VERSION  2  OCTOBER 1976  INCLUDING ERRATA  OCTOBER 1976     POI04806
C                                                                       POI04807
C         DOCUMENTATION FOR THIS PROGRAM IS GIVEN IN                    POI04808
C                                                                       POI04809
C        EFFICIENT FORTRAN SUBPROGRAMS FOR THE SOLUTION OF              POI04810
C            ELLIPTIC PARTIAL DIFFERENTIAL EQUATIONS                    POI04811
C                                                                       POI04812
C                              BY                                       POI04813
C                                                                       POI04814
C          PAUL SWARZTRAUBER   AND  ROLAND SWEET                        POI04815
C                                                                       POI04816
C             TECHNICAL NOTE TN/IA-109   JULY 1975                      POI04817
C                                                                       POI04818
C       NATIONAL CENTER FOR ATMOSPHERIC RESEARCH  BOULDER,COLORADO 80307POI04819
C                                                                       POI04820
C        WHICH IS SPONSORED BY THE NATIONAL SCIENCE FOUNDATION          POI04821
C                                                                       POI04822
C***********************************************************************POI04823
C                                                                       POI04824
C                                                                       POI04825
C     PROGRAM TO ILLUSTRATE THE USE OF PWSPLR.                          POI04826
C                                                                       POI04827
      DIMENSION       F(100,50)  ,BDC(51)    ,BDD(51)    ,W(702)     ,  POI04828
     1                R(51)      ,THETA(49)                             POI04829
C                                                                       POI04830
C     FROM DIMENSION STATEMENT WE GET VALUE OF IDIMF.  ALSO NOTE THAT W POI04831
C     IS DIMENSIONED 6*(N+1) + 8*(M+1).                                 POI04832
C                                                                       POI04833
      IDIMF = 100                                                       POI04834
      INTL = 0                                                          POI04835
      A = 0.                                                            POI04836
      B = 1.                                                            POI04837
      M = 50                                                            POI04838
      MBDCND = 5                                                        POI04839
      C = 0.                                                            POI04840
      PI = 3.14159265358979                                             POI04841
      D = PI/2.                                                         POI04842
      N = 48                                                            POI04843
      NBDCND = 3                                                        POI04844
      ELMBDA = 0.                                                       POI04845
C                                                                       POI04846
C     AUXILIARY QUANTITIES.                                             POI04847
C                                                                       POI04848
      MP1 = M+1                                                         POI04849
      NP1 = N+1                                                         POI04850
C                                                                       POI04851
C     GENERATE AND STORE GRID POINTS FOR THE PURPOSE OF COMPUTING       POI04852
C     BOUNDARY DATA AND THE RIGHT SIDE OF THE POISSON EQUATION.         POI04853
C                                                                       POI04854
      DO 101 I=1,MP1                                                    POI04855
         R(I) = FLOAT(I-1)/50.                                          POI04856
  101 CONTINUE                                                          POI04857
      DO 102 J=1,NP1                                                    POI04858
         THETA(J) = FLOAT(J-1)*PI/96.                                   POI04859
  102 CONTINUE                                                          POI04860
C                                                                       POI04861
C     GENERATE BOUNDARY DATA.                                           POI04862
C                                                                       POI04863
      DO 103 I=1,MP1                                                    POI04864
         BDC(I) = 0.                                                    POI04865
         BDD(I) = 0.                                                    POI04866
  103 CONTINUE                                                          POI04867
C                                                                       POI04868
C     BDA AND BDB ARE DUMMY VARIABLES.                                  POI04869
C                                                                       POI04870
      DO 104 J=1,NP1                                                    POI04871
         F(MP1,J) = 1.-COS(4.*THETA(J))                                 POI04872
  104 CONTINUE                                                          POI04873
C                                                                       POI04874
C     GENERATE RIGHT SIDE OF EQUATION.                                  POI04875
C                                                                       POI04876
      DO 106 I=1,M                                                      POI04877
         DO 105 J=1,NP1                                                 POI04878
            F(I,J) = 16.*R(I)**2                                        POI04879
  105    CONTINUE                                                       POI04880
  106 CONTINUE                                                          POI04881
      CALL PWSPLR (INTL,A,B,M,MBDCND,BDA,BDB,C,D,N,NBDCND,BDC,BDD,      POI04882
     1             ELMBDA,F,IDIMF,PERTRB,IERROR,W)                      POI04883
C                                                                       POI04884
C     COMPUTE DISCRETIZATION ERROR.  THE EXACT SOLUTION IS              POI04885
C                U(R,THETA) = R**4*(1 - COS(4*THETA))                   POI04886
C                                                                       POI04887
      ERR = 0.                                                          POI04888
      DO 108 I=1,MP1                                                    POI04889
         DO 107 J=1,NP1                                                 POI04890
            Z = ABS(F(I,J)-R(I)**4*(1.-COS(4.*THETA(J))))               POI04891
            IF (Z .GT. ERR) ERR = Z                                     POI04892
  107    CONTINUE                                                       POI04893
  108 CONTINUE                                                          POI04894
      PRINT 1001 , IERROR,ERR                                           POI04895
      STOP                                                              POI04896
C                                                                       POI04897
C                                                                       POI04898
C                                                                       POI04899
 1001 FORMAT (///9H IERROR =,I2,10X,22HDISCRETIZATION ERROR =,E12.5)    POI04900
C                                                                       POI04901
      END                                                               POI04902
