C     PROGRAM XAMPLE                                                    POI04904
C                                                                       POI04905
C***********************************************************************POI04906
C                                                                       POI04907
C          VERSION  2  OCTOBER 1976  INCLUDING ERRATA  OCTOBER 1976     POI04908
C                                                                       POI04909
C         DOCUMENTATION FOR THIS PROGRAM IS GIVEN IN                    POI04910
C                                                                       POI04911
C        EFFICIENT FORTRAN SUBPROGRAMS FOR THE SOLUTION OF              POI04912
C            ELLIPTIC PARTIAL DIFFERENTIAL EQUATIONS                    POI04913
C                                                                       POI04914
C                              BY                                       POI04915
C                                                                       POI04916
C          PAUL SWARZTRAUBER   AND  ROLAND SWEET                        POI04917
C                                                                       POI04918
C             TECHNICAL NOTE TN/IA-109   JULY 1975                      POI04919
C                                                                       POI04920
C       NATIONAL CENTER FOR ATMOSPHERIC RESEARCH  BOULDER,COLORADO 80307POI04921
C                                                                       POI04922
C        WHICH IS SPONSORED BY THE NATIONAL SCIENCE FOUNDATION          POI04923
C                                                                       POI04924
C***********************************************************************POI04925
C                                                                       POI04926
C                                                                       POI04927
C     PROGRAM TO ILLUSTRATE THE USE OF PWSCYL.                          POI04928
C                                                                       POI04929
      DIMENSION       F(75,105)  ,BDA(101)   ,BDB(101)   ,BDC(51)    ,  POI04930
     1                BDD(51)    ,W(1014)    ,R(51)      ,Z(101)        POI04931
C                                                                       POI04932
C     FROM DIMENSION STATEMENT WE GET VALUE OF IDIMF.  ALSO NOTE THAT W POI04933
C     IS DIMENSIONED 6*(N+1) + 8*(M+1).                                 POI04934
C                                                                       POI04935
      IDIMF = 75                                                        POI04936
      INTL = 0                                                          POI04937
      A = 0.                                                            POI04938
      B = 1.                                                            POI04939
      M = 50                                                            POI04940
      MBDCND = 6                                                        POI04941
      C = 0.                                                            POI04942
      D = 1.                                                            POI04943
      N = 100                                                           POI04944
      NBDCND = 3                                                        POI04945
      ELMBDA = 0.                                                       POI04946
C                                                                       POI04947
C     AUXILIARY QUANTITIES.                                             POI04948
C                                                                       POI04949
      MP1 = M+1                                                         POI04950
      NP1 = N+1                                                         POI04951
C                                                                       POI04952
C     GENERATE AND STORE GRID POINTS FOR THE PURPOSE OF COMPUTING       POI04953
C     BOUNDARY DATA AND THE RIGHT SIDE OF THE POISSON EQUATION.         POI04954
C                                                                       POI04955
      DO 101 I=1,MP1                                                    POI04956
         R(I) = FLOAT(I-1)/50.                                          POI04957
  101 CONTINUE                                                          POI04958
      DO 102 J=1,NP1                                                    POI04959
         Z(J) = FLOAT(J-1)/100.                                         POI04960
  102 CONTINUE                                                          POI04961
C                                                                       POI04962
C     GENERATE BOUNDARY DATA.                                           POI04963
C                                                                       POI04964
      DO 103 J=1,NP1                                                    POI04965
         BDB(J) = 4.*Z(J)**4                                            POI04966
  103 CONTINUE                                                          POI04967
      DO 104 I=1,MP1                                                    POI04968
         BDC(I) = 0.                                                    POI04969
         BDD(I) = 4.*R(I)**4                                            POI04970
  104 CONTINUE                                                          POI04971
C                                                                       POI04972
C     BDA IS A DUMMY VARIABLE.                                          POI04973
C                                                                       POI04974
C                                                                       POI04975
C     GENERATE RIGHT SIDE OF EQUATION.                                  POI04976
C                                                                       POI04977
      DO 106 I=1,MP1                                                    POI04978
         DO 105 J=1,NP1                                                 POI04979
            F(I,J) = 4.*R(I)**2*Z(J)**2*(4.*Z(J)**2+3.*R(I)**2)         POI04980
  105    CONTINUE                                                       POI04981
  106 CONTINUE                                                          POI04982
      CALL PWSCYL (INTL,A,B,M,MBDCND,BDA,BDB,C,D,N,NBDCND,BDC,BDD,      POI04983
     1             ELMBDA,F,IDIMF,PERTRB,IERROR,W)                      POI04984
C                                                                       POI04985
C     COMPUTE DISCRETIZATION ERROR BY MINIMIZING OVER ALL A THE FUNCTIONPOI04986
C     NORM(F(I,J) - A*1 - U(R(I),Z(J))).  THE EXACT SOLUTION IS         POI04987
C                U(R,Z) = (R*Z)**4 + ARBITRARY CONSTANT.                POI04988
C                                                                       POI04989
      X = 0.                                                            POI04990
      DO 108 I=1,MP1                                                    POI04991
         DO 107 J=1,NP1                                                 POI04992
            X = X+F(I,J)-(R(I)*Z(J))**4                                 POI04993
  107    CONTINUE                                                       POI04994
  108 CONTINUE                                                          POI04995
      X = X/FLOAT(NP1*MP1)                                              POI04996
      DO 110 I=1,MP1                                                    POI04997
         DO 109 J=1,NP1                                                 POI04998
            F(I,J) = F(I,J)-X                                           POI04999
  109    CONTINUE                                                       POI05000
  110 CONTINUE                                                          POI05001
      ERR = 0.                                                          POI05002
      DO 112 I=1,MP1                                                    POI05003
         DO 111 J=1,NP1                                                 POI05004
            X = ABS(F(I,J)-(R(I)*Z(J))**4)                              POI05005
            IF (X .GT. ERR) ERR = X                                     POI05006
  111    CONTINUE                                                       POI05007
  112 CONTINUE                                                          POI05008
      PRINT 1001 , PERTRB,IERROR,ERR                                    POI05009
      STOP                                                              POI05010
C                                                                       POI05011
C                                                                       POI05012
C                                                                       POI05013
 1001 FORMAT (///9H PERTRB =,E12.5,10X,8HIERROR =,I2,10X,               POI05014
     1        22HDISCRETIZATION ERROR =,E12.5)                          POI05015
C                                                                       POI05016
      END                                                               POI05017
