C     PROGRAM XAMPLE                                                    POI05115
C                                                                       POI05116
C***********************************************************************POI05117
C                                                                       POI05118
C          VERSION  2  OCTOBER 1976  INCLUDING ERRATA  OCTOBER 1976     POI05119
C                                                                       POI05120
C         DOCUMENTATION FOR THIS PROGRAM IS GIVEN IN                    POI05121
C                                                                       POI05122
C        EFFICIENT FORTRAN SUBPROGRAMS FOR THE SOLUTION OF              POI05123
C            ELLIPTIC PARTIAL DIFFERENTIAL EQUATIONS                    POI05124
C                                                                       POI05125
C                              BY                                       POI05126
C                                                                       POI05127
C          PAUL SWARZTRAUBER   AND  ROLAND SWEET                        POI05128
C                                                                       POI05129
C             TECHNICAL NOTE TN/IA-109   JULY 1975                      POI05130
C                                                                       POI05131
C       NATIONAL CENTER FOR ATMOSPHERIC RESEARCH  BOULDER,COLORADO 80307POI05132
C                                                                       POI05133
C        WHICH IS SPONSORED BY THE NATIONAL SCIENCE FOUNDATION          POI05134
C                                                                       POI05135
C***********************************************************************POI05136
C                                                                       POI05137
C                                                                       POI05138
C     PROGRAM TO ILLUSTRATE THE USE OF PWSCSP                           POI05139
C                                                                       POI05140
      DIMENSION       F(48,33)   ,BDTF(33)   ,W(902)     ,R(33)      ,  POI05141
     1                THETA(48)                                         POI05142
C                                                                       POI05143
C     THE VALUE OF IDIMF IS THE FIRST DIMENSION OF F. SINCE N=31,L=32   POI05144
C     THEREFORE K=5 AND W IS DIMENSIONED 2(L+1)(K-1)+6(M+N)+MAX(4N,6M)  POI05145
C     +14 = 902                                                         POI05146
C                                                                       POI05147
      PI = 3.141592653589793                                            POI05148
      INTL = 0                                                          POI05149
      TS = 0.                                                           POI05150
      TF = PI/2.                                                        POI05151
      M = 36                                                            POI05152
      MBDCND = 6                                                        POI05153
      RS = 0.                                                           POI05154
      RF = 1.                                                           POI05155
      N = 32                                                            POI05156
      NBDCND = 5                                                        POI05157
      ELMBDA = 0.                                                       POI05158
      IDIMF = 48                                                        POI05159
C                                                                       POI05160
C     GENERATE AND STORE GRID POINTS FOR THE PURPOSE OF COMPUTING THE   POI05161
C     BOUNDARY DATA AND THE RIGHT SIDE OF THE EQUATION.                 POI05162
C                                                                       POI05163
      MP1 = M+1                                                         POI05164
      DTHETA = TF/FLOAT(M)                                              POI05165
      DO 101 I=1,MP1                                                    POI05166
         THETA(I) = FLOAT(I-1)*DTHETA                                   POI05167
  101 CONTINUE                                                          POI05168
      NP1 = N+1                                                         POI05169
      DR = 1./FLOAT(N)                                                  POI05170
      DO 102 J=1,NP1                                                    POI05171
         R(J) = FLOAT(J-1)*DR                                           POI05172
  102 CONTINUE                                                          POI05173
C                                                                       POI05174
C     GENERATE NORMAL DERIVATIVE DATA AT EQUATOR                        POI05175
C                                                                       POI05176
      DO 103 J=1,NP1                                                    POI05177
         BDTF(J) = 0.                                                   POI05178
  103 CONTINUE                                                          POI05179
C                                                                       POI05180
C     COMPUTE BOUNDARY DATA ON THE SURFACE OF THE SPHERE                POI05181
C                                                                       POI05182
      DO 104 I=1,MP1                                                    POI05183
         F(I,N+1) = COS(THETA(I))**4                                    POI05184
  104 CONTINUE                                                          POI05185
C                                                                       POI05186
C     COMPUTE RIGHT SIDE OF EQUATION                                    POI05187
C                                                                       POI05188
      DO 106 I=1,MP1                                                    POI05189
         CI4 = 12.*COS(THETA(I))**2                                     POI05190
         DO 105 J=1,N                                                   POI05191
            F(I,J) = CI4*R(J)**2                                        POI05192
  105    CONTINUE                                                       POI05193
  106 CONTINUE                                                          POI05194
C                                                                       POI05195
      CALL PWSCSP (INTL,TS,TF,M,MBDCND,BDTS,BDTF,RS,RF,N,NBDCND,BDRS,   POI05196
     1             BDRF,ELMBDA,F,IDIMF,PERTRB,IERROR,W)                 POI05197
C                                                                       POI05198
C     COMPUTE DISCRETIZATION ERROR                                      POI05199
C                                                                       POI05200
      ERR = 0.                                                          POI05201
      DO 108 I=1,MP1                                                    POI05202
         CI4 = COS(THETA(I))**4                                         POI05203
         DO 107 J=1,N                                                   POI05204
            Z = ABS(F(I,J)-CI4*R(J)**4)                                 POI05205
            IF (Z .GT. ERR) ERR = Z                                     POI05206
  107    CONTINUE                                                       POI05207
  108 CONTINUE                                                          POI05208
      PRINT 1001 , IERROR,ERR                                           POI05209
C                                                                       POI05210
C     THE FOLLOWING PROGRAM ILLUSTRATES THE USE OF PWSCSP TO SOLVE      POI05211
C     A THREE DIMENSIONAL PROBLEM WHICH HAS LONGITUDNAL DEPENDENCE      POI05212
C                                                                       POI05213
      MBDCND = 2                                                        POI05214
      NBDCND = 1                                                        POI05215
      DPHI = PI/72.                                                     POI05216
      ELMBDA = -2.*(1.-COS(DPHI))/DPHI**2                               POI05217
C                                                                       POI05218
C     COMPUTE BOUNDARY DATA ON THE SURFACE OF THE SPHERE                POI05219
C                                                                       POI05220
      DO 109 I=1,MP1                                                    POI05221
         F(I,N+1) = SIN(THETA(I))                                       POI05222
  109 CONTINUE                                                          POI05223
C                                                                       POI05224
C     COMPUTE RIGHT SIDE OF THE EQUATION                                POI05225
C                                                                       POI05226
      DO 111 J=1,N                                                      POI05227
         DO 110 I=1,MP1                                                 POI05228
            F(I,J) = 0.                                                 POI05229
  110    CONTINUE                                                       POI05230
  111 CONTINUE                                                          POI05231
C                                                                       POI05232
      CALL PWSCSP (INTL,TS,TF,M,MBDCND,BDTS,BDTF,RS,RF,N,NBDCND,BDRS,   POI05233
     1             BDRF,ELMBDA,F,IDIMF,PERTRB,IERROR,W)                 POI05234
C                                                                       POI05235
C     COMPUTE DISCRETIZATION ERROR   (FOURIER COEFFICIENTS)             POI05236
C                                                                       POI05237
      ERR = 0                                                           POI05238
      DO 113 I=1,MP1                                                    POI05239
         SI = SIN(THETA(I))                                             POI05240
         DO 112 J=1,NP1                                                 POI05241
            Z = ABS(F(I,J)-R(J)*SI)                                     POI05242
            IF (Z .GT. ERR) ERR = Z                                     POI05243
  112    CONTINUE                                                       POI05244
  113 CONTINUE                                                          POI05245
C                                                                       POI05246
      PRINT 1001 , IERROR,ERR                                           POI05247
      STOP                                                              POI05248
C                                                                       POI05249
C                                                                       POI05250
C                                                                       POI05251
 1001 FORMAT (///9H IERROR =,I3,10X,22HDISCRETIZATION ERROR =,E12.4)    POI05252
C                                                                       POI05253
      END                                                               POI05254
