C     PROGRAM XAMPLE                                                    POI05256
C                                                                       POI05257
C***********************************************************************POI05258
C                                                                       POI05259
C          VERSION  3  OCTOBER 1978                                     POI05260
C                                                                       POI05261
C         DOCUMENTATION FOR THIS PROGRAM IS GIVEN IN                    POI05262
C                                                                       POI05263
C        EFFICIENT FORTRAN SUBPROGRAMS FOR THE SOLUTION OF              POI05264
C            ELLIPTIC PARTIAL DIFFERENTIAL EQUATIONS                    POI05265
C                                                                       POI05266
C                              BY                                       POI05267
C                                                                       POI05268
C          PAUL SWARZTRAUBER   AND  ROLAND SWEET                        POI05269
C                                                                       POI05270
C             TECHNICAL NOTE TN/IA-109   JULY 1975                      POI05271
C                                                                       POI05272
C       NATIONAL CENTER FOR ATMOSPHERIC RESEARCH  BOULDER,COLORADO 80307POI05273
C                                                                       POI05274
C        WHICH IS SPONSORED BY THE NATIONAL SCIENCE FOUNDATION          POI05275
C                                                                       POI05276
C***********************************************************************POI05277
C                                                                       POI05278
C                                                                       POI05279
C     PROGRAM TO ILLUSTRATE THE USE OF SUBROUTINE POIS.                 POI05280
C                                                                       POI05281
      DIMENSION       F(25,130)  ,A(20)      ,B(20)      ,C(20)      ,  POI05282
     1                W(820)     ,X(20)      ,Y(120)                    POI05283
C                                                                       POI05284
C     FROM DIMENSION STATEMENT WE GET VALUE OF IDIMY.  ALSO NOTE THAT W POI05285
C     IS DIMENSIONED 6*(N+1) + 5*(M+1).                                 POI05286
C                                                                       POI05287
      IDIMY = 25                                                        POI05288
      IFLG = 0                                                          POI05289
      MPEROD = 1                                                        POI05290
      M = 20                                                            POI05291
      DELTAX = 1./FLOAT(M)                                              POI05292
      NPEROD = 0                                                        POI05293
      N = 120                                                           POI05294
      PI = 3.14159265358979                                             POI05295
      DELTAY = 2.*PI/FLOAT(N)                                           POI05296
C                                                                       POI05297
C     GENERATE AND STORE GRID POINTS FOR THE PURPOSE OF COMPUTING       POI05298
C     COEFFICIENTS AND RIGHT SIDE OF EQUATION.                          POI05299
C                                                                       POI05300
      DO 101 I=1,M                                                      POI05301
         X(I) = FLOAT(I-1)*DELTAX                                       POI05302
  101 CONTINUE                                                          POI05303
      DO 102 J=1,120                                                    POI05304
         Y(J) = -PI+FLOAT(J-1)*DELTAY                                   POI05305
  102 CONTINUE                                                          POI05306
C                                                                       POI05307
C     GENERATE COEFFICIENTS.                                            POI05308
C                                                                       POI05309
      S = (DELTAY/DELTAX)**2                                            POI05310
      T = S*DELTAX                                                      POI05311
      A(1) = 0.                                                         POI05312
      B(1) = -2.*S                                                      POI05313
      C(1) = 2.*S                                                       POI05314
      DO 103 I=2,M                                                      POI05315
         A(I) = (1.+X(I))**2*S+(1.+X(I))*T                              POI05316
         C(I) = (1.+X(I))**2*S-(1.+X(I))*T                              POI05317
         B(I) = -2.*(1.+X(I))**2*S                                      POI05318
  103 CONTINUE                                                          POI05319
      C(M) = 0.                                                         POI05320
C                                                                       POI05321
C     GENERATE RIGHT SIDE OF EQUATION FOR I = 1 SHOWING INTRODUCTION OF POI05322
C     BOUNDARY DATA.                                                    POI05323
C                                                                       POI05324
      DYSQ = DELTAY**2                                                  POI05325
      DO 104 J=1,N                                                      POI05326
         F(1,J) = DYSQ*(11.+8./DELTAX)*SIN(Y(J))                        POI05327
  104 CONTINUE                                                          POI05328
C                                                                       POI05329
C     GENERATE RIGHT SIDE.                                              POI05330
C                                                                       POI05331
      MM1 = M-1                                                         POI05332
      DO 106 I=2,MM1                                                    POI05333
         DO 105 J=1,N                                                   POI05334
            F(I,J) = DYSQ*3.*(1.+X(I))**4*SIN(Y(J))                     POI05335
  105    CONTINUE                                                       POI05336
  106 CONTINUE                                                          POI05337
C                                                                       POI05338
C     GENERATE RIGHT SIDE FOR I = M SHOWING INTRODUCTION OF             POI05339
C     BOUNDARY DATA.                                                    POI05340
C                                                                       POI05341
      DO 107 J=1,N                                                      POI05342
         F(M,J) = DYSQ*(3.*(1.+X(M))**4-16.*((1.+X(M))/DELTAX)**2+      POI05343
     1                                   16.*(1.+X(M))/DELTAX)*SIN(Y(J))POI05344
  107 CONTINUE                                                          POI05345
      CALL POIS (IFLG,NPEROD,N,MPEROD,M,A,B,C,IDIMY,F,IERROR,W)         POI05346
C                                                                       POI05347
C     COMPUTE DISCRETIATION ERROR.  THE EXACT SOLUTION IS               POI05348
C                   U(X,Y) = (1+X)**4*SIN(Y)                            POI05349
C                                                                       POI05350
      ERR = 0.                                                          POI05351
      DO 109 I=1,M                                                      POI05352
         DO 108 J=1,N                                                   POI05353
            Z = ABS(F(I,J)-(1.+X(I))**4*SIN(Y(J)))                      POI05354
            IF (Z .GT. ERR) ERR = Z                                     POI05355
  108    CONTINUE                                                       POI05356
  109 CONTINUE                                                          POI05357
      PRINT 1001 , IERROR,ERR                                           POI05358
      STOP                                                              POI05359
C                                                                       POI05360
C                                                                       POI05361
C                                                                       POI05362
 1001 FORMAT (///9H IERROR =,I3,10X,22HDISCRETIZATION ERROR =,E12.4)    POI05363
C                                                                       POI05364
      END                                                               POI05365
