C     PROGRAM XAMPLE                                                    POI05367
C                                                                       POI05368
C***********************************************************************POI05369
C                                                                       POI05370
C          VERSION  2  OCTOBER 1976  INCLUDING ERRATA  OCTOBER 1976     POI05371
C                                                                       POI05372
C         DOCUMENTATION FOR THIS PROGRAM IS GIVEN IN                    POI05373
C                                                                       POI05374
C        EFFICIENT FORTRAN SUBPROGRAMS FOR THE SOLUTION OF              POI05375
C            ELLIPTIC PARTIAL DIFFERENTIAL EQUATIONS                    POI05376
C                                                                       POI05377
C                              BY                                       POI05378
C                                                                       POI05379
C          PAUL SWARZTRAUBER   AND  ROLAND SWEET                        POI05380
C                                                                       POI05381
C             TECHNICAL NOTE TN/IA-109   JULY 1975                      POI05382
C                                                                       POI05383
C       NATIONAL CENTER FOR ATMOSPHERIC RESEARCH  BOULDER,COLORADO 80307POI05384
C                                                                       POI05385
C        WHICH IS SPONSORED BY THE NATIONAL SCIENCE FOUNDATION          POI05386
C                                                                       POI05387
C***********************************************************************POI05388
C                                                                       POI05389
C                                                                       POI05390
C     PROGRAM TO ILLUSTRATE THE USE OF BLKTRI                           POI05391
C                                                                       POI05392
      DIMENSION       Y(75,105)  ,AM(75)     ,BM(75)     ,CM(75)     ,  POI05393
     1                AN(105)    ,BN(105)    ,CN(105)    ,W(942)     ,  POI05394
     2                S(75)      ,T(105)                                POI05395
C                                                                       POI05396
C     THE VALUE OF IDIMY IS THE FIRST DIMENSION OF Y. SINCE NP=1 W IS   POI05397
C     DIMENSIONED 2*(N+1)*(LOG2(N+1)-1)+2+MAX(2*N,6*M)=942              POI05398
C                                                                       POI05399
      IFLG = 0                                                          POI05400
      NP = 1                                                            POI05401
      N = 63                                                            POI05402
      MP = 1                                                            POI05403
      M = 50                                                            POI05404
      IDIMY = 75                                                        POI05405
C                                                                       POI05406
C     GENERATE AND STORE GRID POINTS FOR THE PURPOSE OF COMPUTING THE   POI05407
C     COEFFICIENTS AND THE ARRAY Y.                                     POI05408
C                                                                       POI05409
      DELTAS = 1./FLOAT(M+1)                                            POI05410
      DO 101 I=1,M                                                      POI05411
         S(I) = FLOAT(I)*DELTAS                                         POI05412
  101 CONTINUE                                                          POI05413
      DELTAT = 1./FLOAT(N+1)                                            POI05414
      DO 102 J=1,N                                                      POI05415
         T(J) = FLOAT(J)*DELTAT                                         POI05416
  102 CONTINUE                                                          POI05417
C                                                                       POI05418
C     COMPUTE THE COEFFICIENTS AM,BM,CM CORRESPONDING TO THE S DIRECTIONPOI05419
C                                                                       POI05420
      HDS = DELTAS/2.                                                   POI05421
      TDS = DELTAS+DELTAS                                               POI05422
      DO 103 I=1,M                                                      POI05423
         TEMP1 = 1./(S(I)*TDS)                                          POI05424
         TEMP2 = 1./((S(I)-HDS)*TDS)                                    POI05425
         TEMP3 = 1./((S(I)+HDS)*TDS)                                    POI05426
         AM(I) = TEMP1*TEMP2                                            POI05427
         CM(I) = TEMP1*TEMP3                                            POI05428
         BM(I) = -(AM(I)+CM(I))                                         POI05429
  103 CONTINUE                                                          POI05430
C                                                                       POI05431
C     COMPUTE THE COEFFICIENTS AN,BN,CN CORRESPONDING TO THE T DIRECTIONPOI05432
C                                                                       POI05433
      HDT = DELTAT/2.                                                   POI05434
      TDT = DELTAT+DELTAT                                               POI05435
      DO 104 J=1,N                                                      POI05436
         TEMP1 = 1./(T(J)*TDT)                                          POI05437
         TEMP2 = 1./((T(J)-HDT)*TDT)                                    POI05438
         TEMP3 = 1./((T(J)+HDT)*TDT)                                    POI05439
         AN(J) = TEMP1*TEMP2                                            POI05440
         CN(J) = TEMP1*TEMP3                                            POI05441
         BN(J) = -(AN(J)+CN(J))                                         POI05442
  104 CONTINUE                                                          POI05443
C                                                                       POI05444
C     COMPUTE RIGHT SIDE OF EQUATION                                    POI05445
C                                                                       POI05446
      DO 106 J=1,N                                                      POI05447
         DO 105 I=1,M                                                   POI05448
            Y(I,J) = 3.75*S(I)*T(J)*(S(I)**4+T(J)**4)                   POI05449
  105    CONTINUE                                                       POI05450
  106 CONTINUE                                                          POI05451
C                                                                       POI05452
C     INCLUDE NONHOMOGENEOUS BOUNDARY INTO RIGHT SIDE. NOTE THAT THE    POI05453
C     CORNER AT J=N,I=M INCLUDES CONTRIBUTIONS FROM BOTH BOUNDARIES.    POI05454
C                                                                       POI05455
      DO 107 J=1,N                                                      POI05456
         Y(M,J) = Y(M,J)-CM(M)*T(J)**5                                  POI05457
  107 CONTINUE                                                          POI05458
      DO 108 I=1,M                                                      POI05459
         Y(I,N) = Y(I,N)-CN(N)*S(I)**5                                  POI05460
  108 CONTINUE                                                          POI05461
C                                                                       POI05462
  109 CALL BLKTRI (IFLG,NP,N,AN,BN,CN,MP,M,AM,BM,CM,IDIMY,Y,IERROR,W)   POI05463
      IFLG = IFLG+1                                                     POI05464
      IF (IFLG-1) 109,109,110                                           POI05465
C                                                                       POI05466
C     COMPUTE DISCRETIZATION ERROR                                      POI05467
C                                                                       POI05468
  110 ERR = 0.                                                          POI05469
      DO 112 J=1,N                                                      POI05470
         DO 111 I=1,M                                                   POI05471
            Z = ABS(Y(I,J)-(S(I)*T(J))**5)                              POI05472
            IF (Z .GT. ERR) ERR = Z                                     POI05473
  111    CONTINUE                                                       POI05474
  112 CONTINUE                                                          POI05475
      PRINT 1001 , IERROR,ERR                                           POI05476
      STOP                                                              POI05477
C                                                                       POI05478
C                                                                       POI05479
C                                                                       POI05480
 1001 FORMAT (///9H IERROR =,I3,10X,22HDISCRETIZATION ERROR =,E12.4)    POI05481
C                                                                       POI05482
      END                                                               POI05483
